#include <iostream>
#include <windows.h>
#include <sql.h>
#include <sqlext.h>
#include <cstdint>  // For int64_t
#include <thread>

// Constants for timeout values
#define DEFAULT_CONNECT_TIMEOUT 5
#define DEFAULT_RESPONSE_TIMEOUT 30

// Assuming DBH is a structure containing the connection handle and result
struct DBHStruct {
    SQLHENV henv;
    SQLHDBC hdbc;
    SQLRETURN res;
    SQLCHAR errors[1024];
} DBH[2];

// Function to set SQL error
void SetSQLError(SQLSMALLINT handleType, SQLHANDLE handle, SQLCHAR* errorMsg, size_t size) {
    SQLCHAR sqlState[6];
    SQLINTEGER nativeError;
    SQLCHAR messageText[256];
    SQLSMALLINT textLength;

    if (SQLGetDiagRec(handleType, handle, 1, sqlState, &nativeError, messageText, sizeof(messageText), &textLength) == SQL_SUCCESS) {
        std::cerr << "SQL Error: " << messageText << std::endl;
        strncpy((char*)errorMsg, (char*)messageText, size);
    }
    else {
        std::cerr << "Failed to retrieve SQL error information" << std::endl;
    }
}

int run(int i)
{
    if (!SQL_SUCCEEDED(SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &DBH[i].henv))) {
        SetSQLError(SQL_HANDLE_ENV, DBH[i].henv, DBH[i].errors, sizeof(DBH[i].errors));
        return -1;
    }

    // Set ODBC version
    if (!SQL_SUCCEEDED(SQLSetEnvAttr(DBH[i].henv, SQL_ATTR_ODBC_VERSION, (SQLPOINTER)SQL_OV_ODBC3, 0))) {
        SetSQLError(SQL_HANDLE_ENV, DBH[i].henv, DBH[i].errors, sizeof(DBH[i].errors));
        SQLFreeHandle(SQL_HANDLE_ENV, DBH[i].henv);  // Clean up
        return -1;
    }

    // Allocate connection handle
    if (!SQL_SUCCEEDED(SQLAllocHandle(SQL_HANDLE_DBC, DBH[i].henv, &DBH[i].hdbc))) {
        SetSQLError(SQL_HANDLE_ENV, DBH[i].henv, DBH[i].errors, sizeof(DBH[i].errors));
        SQLFreeHandle(SQL_HANDLE_ENV, DBH[i].henv);  // Clean up
        return -1;
    }

    // Set connection attributes
    SQLRETURN res;
    res = SQLSetConnectAttr(DBH[i].hdbc, SQL_ATTR_LOGIN_TIMEOUT, (SQLPOINTER)(uintptr_t)DEFAULT_CONNECT_TIMEOUT, SQL_IS_UINTEGER);
    if (!SQL_SUCCEEDED(res)) {
        SetSQLError(SQL_HANDLE_DBC, DBH[i].hdbc, DBH[i].errors, sizeof(DBH[i].errors));
        SQLFreeHandle(SQL_HANDLE_DBC, DBH[i].hdbc);  // Clean up
        SQLFreeHandle(SQL_HANDLE_ENV, DBH[i].henv);  // Clean up
        return -1;
    }

    res = SQLSetConnectAttr(DBH[i].hdbc, SQL_ATTR_CONNECTION_TIMEOUT, (SQLPOINTER)(uintptr_t)DEFAULT_RESPONSE_TIMEOUT, SQL_IS_UINTEGER);
    if (!SQL_SUCCEEDED(res)) {
        SetSQLError(SQL_HANDLE_DBC, DBH[i].hdbc, DBH[i].errors, sizeof(DBH[i].errors));
        SQLFreeHandle(SQL_HANDLE_DBC, DBH[i].hdbc);  // Clean up
        SQLFreeHandle(SQL_HANDLE_ENV, DBH[i].henv);  // Clean up
        return -1;
    }

    res = SQLSetConnectAttr(DBH[i].hdbc, SQL_ATTR_AUTOCOMMIT, (SQLPOINTER)SQL_AUTOCOMMIT_ON, SQL_IS_UINTEGER);
    if (!SQL_SUCCEEDED(res)) {
        SetSQLError(SQL_HANDLE_DBC, DBH[i].hdbc, DBH[i].errors, sizeof(DBH[i].errors));
        SQLFreeHandle(SQL_HANDLE_DBC, DBH[i].hdbc);  // Clean up
        SQLFreeHandle(SQL_HANDLE_ENV, DBH[i].henv);  // Clean up
        return -1;
    }

    // Connect to data source
    try {
        SQLCHAR name[] = "MySqlANSI";
        SQLCHAR user[] = "root";
        SQLCHAR pass[] = "gwtQhndT65pNzyZClJMS";

        DBH[i].res = SQLConnect(DBH[i].hdbc, name, SQL_NTS, user, SQL_NTS, pass, SQL_NTS);
        if (!SQL_SUCCEEDED(DBH[i].res)) {
            throw std::runtime_error("Failed to connect to data source");
        }
        std::cout << "Connected to the database successfully" << std::endl;
    }
    catch (const std::exception& e) {
        std::cerr << "Exception: " << e.what() << std::endl;
        SetSQLError(SQL_HANDLE_DBC, DBH[i].hdbc, DBH[i].errors, sizeof(DBH[i].errors));
        SQLFreeHandle(SQL_HANDLE_DBC, DBH[i].hdbc);  // Clean up
        SQLFreeHandle(SQL_HANDLE_ENV, DBH[i].henv);  // Clean up
        return -1;
    }

    // Your further logic here

    // Clean up handles
    SQLFreeHandle(SQL_HANDLE_DBC, DBH[i].hdbc);
    SQLFreeHandle(SQL_HANDLE_ENV, DBH[i].henv);
}

int main() {
    // Initialize ODBC environment handle
    std::thread th1 ([]() {
        
        run(0);

        });

    std::thread th2([]() {

        run(1);

        });

    th1.join();
    th2.join();



    return 0;
}
